/**********************************************************************************************/
/* FBL_ThreadNative.h																	  	  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/
/*
	FBL_ThreadNative -- abstract class which specify interface of native OS thread class.

	FBL_Thread is wrapper to abstract FBL_ThreadNative class which specify interface of native 
	OS thread class. We use wrapping but not inheritance becuse Valentina SDK must be	  
	very flexiable -- work on different platforms and with different app frameworks.

	So if you have own framework which Valentina do not support on default you just need 
	subclass FBL_ThreadNative and provide own implementaition of thread class. You will
	use multiple inheritance. Look on FBL_PPThread as example.
*/

#ifndef __FBL_ThreadNative_h
	#define __FBL_ThreadNative_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL_Task.h>
#	include <VSDK/FBL_Globals.h>
#else
#	include <FBL_Task.h>
#	include <FBL_Globals.h>
#endif // VSDK_FRAMEWORK_STYLE

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class FBL_Thread;
class FBL_ThreadNative;


/**********************************************************************************************/
#ifdef FBL_MAC
void FBL_Yield( void );		// yield immidiatly.
void FBL_TickYield( void );	// yield only if have gone FBL_Task::sMaxTicks.

/** FBL_ModalYield gives the time to another applications on cooperative MacOS 
	even if you have busy loop in a function of signle thread application. */
inline void FBL_ModalYield( void )
{
#if !__MACH__ && !defined(__GNUC__)
	if( gFBL_OsXHere == false )
	{
		// Give time to other applications
		EventRecord macEvent;
		::EventAvail( everyEvent, &macEvent );
	}
#endif

	// Yield to any other thread of this appliation
	FBL_Yield();
}

/** Do Modal yield only when have gone FBL_Task::sMaxTicks. 
	This function probably will be removed later. */
inline void FBL_TickModalYield( void )
{
	ulong Ticks = ::TickCount();
	if( Ticks - FBL_Task::sLastYieldTick > FBL_Task::sMaxTicks ){																		
		FBL_Task::sLastYieldTick = Ticks;							
		FBL_ModalYield();							
	}
}

#endif


/**********************************************************************************************/
#ifndef FBL_MAC
// for preemptive threads no operation:
inline void FBL_Yield( void )			{}	
inline void FBL_TickYield( void )		{}
inline void FBL_ModalYield( void )		{}
inline void FBL_TickModalYield( void ) 	{}
inline ulong TickCount( void )			{ return ::GetTickCount() / 16; }
#endif


/**********************************************************************************************/
/*
	The following functions in normal way should be static functions of class.
	But for provided hierarhy they must be "polymorhic".
	To get this we declare them as global and define body of functions in corresponded cpp file.
*/

void 				FBL_ThreadNative_InitStaticMembers( void ); 
void 				FBL_ThreadNative_DestroyStaticMembers( void ); 

FBL_ThreadNative* 	FBL_ThreadNative_Create( 
						FBL_Thread* inThread, 
						ulong 		inStackSize, 
						ulong 		inFlags ); 
						
ulong 				FBL_GetCurrentThreadID( void );


/**********************************************************************************************/
class FBL_ThreadNative 
{
	public://///////////////////////////////////////////////////////////////////////////////////
	 
							FBL_ThreadNative( FBL_Thread* inFBL_Thread ){
								mFBL_Thread = inFBL_Thread;
							}

virtual						~FBL_ThreadNative( void );	
						

		/* Thread execution: */

							/** Delete this thread of OS. */
virtual void				DeleteThread( void ) { delete this; }

virtual void				Resume( void ) 		= 0;	
virtual void				Suspend( void ) 	= 0;	

virtual ulong				GetID( void ) 		= 0;


	protected:////////////////////////////////////////////////////////////////////////////////// 

		FBL_Thread*			mFBL_Thread;	// pointer on wrapper class FBL_Thread.
};


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif // __FBL_ThreadNative_h
